// Academic Integrity reminder, in the form of a Star Wars
// crawl.

// The array of strings that will contain the text of the 
// academic integrity statement.
String[] txt_file;
float space_width;

// The messiest part of the sketch.  Given a paragraph of text, figure
// out where to put the line breaks and space the words out in each line
// so that the text is justified.
float drawParagraph( String txt, float xl, float yt, float lw, float leading )
{
  float w = textWidth( txt );
  if ( w < lw ) {
    // The whole paragraph fits on a single line.  Don't
    // do any breaking, and also centre the text.
    textAlign( CENTER );
    text( txt, xl + 0.5*lw, yt );
    return yt + leading;
  } else {
    // It's a long line.  Need to break it up.
    textAlign( LEFT );
    String[] wds = splitTokens( txt );

    // Build an array containing the pixel width of each
    // word in the paragraph.
    float[] lens = new float[ wds.length ];
    for ( int idx = 0; idx < wds.length; ++idx ) {
      lens[idx] = textWidth( wds[idx] );
    }

    int istart = 0;
    boolean end = false;

    while ( istart < wds.length ) {
      int iend = istart;
      float curw = 0.0;

      do {
        curw += (lens[iend] + space_width);
        ++iend;
        if ( iend >= wds.length ) {
          end = true;
          break;
        }
      } while ( curw < lw );

      if ( !end ) {
        --iend;
      }

      // Now compute the "real" width in order to figure
      // out the spacing we need.
      curw = 0;
      for ( int idx = istart; idx < iend; ++idx ) {
        curw += lens[idx];
      }

      float sp = (lw-curw) / (iend-istart-1);
      if ( end ) {
        sp = space_width;
      }
      float curx = xl;
      for ( int idx = istart; idx < iend; ++idx ) {
        text( wds[idx], curx, yt );
        curx += lens[ idx ] + sp;
      }

      yt += leading;
      istart = iend;
      if ( end ) {
        break;
      }
    }

    return yt;
  }
}

// Draw a field of random stars in the background.  Later in
// the term we'll learn more about this kind of thing.
void drawStars()
{
  // The use of randomSeed( 0 ) guarantees that we always get
  // the same star field every frame.  If you want to see why
  // that's important, try temporarily commenting out this line.
  // Challenge: get the same behaviour, but with a different
  // random star field each time the program is run.
  
  randomSeed( 0 );
  fill( 255, 200 );
  for ( int idx = 0; idx < 1000; ++idx ) {
    float rad = map( idx, 0, 999, 0.1, 2 );
    ellipse( random( width ), random( height ), rad, rad );
  }
}

void setup()
{
  // Use widescreen 16x9 proportions.
  size( 800, 450, P3D );   
  textFont( createFont( "Helvetica", 18 ) );
  textSize( 18 );
  noStroke();

  // Load the text from an external file.  We'll
  // learn more about this shortly.
  txt_file = loadStrings( "lines.txt" );
  
  // How many pixels wide is a single space?
  space_width = textWidth( " " );
} 

void draw()
{  
  background( 0 );

  drawStars();
  pushMatrix();
  fill( 255, 255, 0 );

  translate( 0.5 * width, 0.5 * height );
  rotateX( 0.4*PI );
  translate( 0.0, 300 - frameCount * 0.2, 0.0 );

  float yt = 0;

  for ( String s : txt_file ) {
    yt = drawParagraph( s, -150.0, yt, 300.0, 22.0 );
  }
  popMatrix();
}